<?php
/**
 * ShopEx licence
 *
 * @copyright  Copyright (c) 2005-2012 ShopEx Technologies Inc. (http://www.shopex.cn)
 * @license  http://ecos.shopex.cn/ ShopEx License
 */

class base_events_dispatcher
{
    /**
     * The registered event listeners.
     *
     * @var array
     */
    protected $listeners = [];

    /**
     * The sorted event listeners.
     *
     * @var array
     */
    protected $sorted = [];

    /**
     * The event firing stack.
     *
     * @var array
     */
    protected $firing = [];

    /**
     * Register an event listener with the dispatcher.
     *
     * @param  string|array  $events
     * @param  mixed  $listener
     * @param  string $sync   sync 同步｜async 异步
     * @param  int  $priority
     * @return void
     */
    public function listen($events, $listener, $sync='sync', $priority = 0)
    {
        foreach ((array) $events as $event)
        {
            $this->listeners[$event][$sync][$priority][] = $listener;
            unset($this->sorted[$event]);
        }
    }

    /**
     * Determine if a given event has listeners.
     *
     * @param  string  $eventName
     * @return bool
     */
    public function hasListeners($eventName)
    {
        return isset($this->listeners[$eventName]);
    }

    /**
     * Fire an event until the first non-null response is returned.
     *
     * @param  string|object  $event
     * @param  array  $params
     * @return mixed
     */
    public function until($event, $params= [])
    {
        return $this->fire($event, $params, true);
    }

    /**
     * 执行异步事件任务
     *
     * @param  string  $event
     * @param  array  $payload
     * @return void
     */
    public function push($event, $listener, $params = [] )
    {
        $this->listen($event, $listener);
        $this->fire($event, $params);
    }

    /**
     * Get the event that is currently firing.
     *
     * @return string
     */
    public function firing()
    {
        return last($this->firing);
    }

    private function __preInitEvents($eventName)
    {
        if( ! $this->listeners )
        {
            $EventService = kernel::single('base_events_service', $eventName);
            $EventService->boot($this);
        }
    }

    /**
     * Fire an event and call the listeners.
     *
     * @param  string|object  $eventName
     * @param  array $params
     * @param  bool  $halt
     * @return array|null
     */
    public function fire($eventName, $params, $halt = false)
    {
        $this->__preInitEvents($eventName);

        $responses = [];

        $this->firing[] = $eventName;

        //执行同步listeners
        foreach ($this->getListeners($eventName)['sync'] as $key=>$listener)
        {
            $callData = $this->parseClassCallable($listener);

            $response = kernel::single($callData[0])->$callData[1]($params);

            if (! is_null($response) && $halt)
            {
                array_pop($this->firing);
                return $response;
            }

            if ($response === false)
            {
                break;
            }

            $responses[] = $response;
        }

        //执行异步listeners , 将异步listeners加入到队列
        foreach ($this->getListeners($eventName)['async'] as $key=>$listener)
        {
            $queueParams['eventParams'] = $params;
            $queueParams['eventName'] = $eventName;
            $queueParams['listener'] = $listener;
            system_queue::instance()->publish('system_tasks_events', 'system_tasks_events', $queueParams);
        }

        array_pop($this->firing);

        return $responses;
    }

    /**
     * Get all of the listeners for a given event name.
     *
     * @param  string  $eventName
     * @return array
     */
    public function getListeners($eventName)
    {
        if (! isset($this->sorted[$eventName]))
        {
            $this->sortListeners($eventName);
        }

        return $this->sorted[$eventName];
    }

    /**
     * Sort the listeners for a given event by priority.
     *
     * @param  string  $eventName
     * @return array
     */
    protected function sortListeners($eventName)
    {
        $this->sorted[$eventName] = [];

        if (isset($this->listeners[$eventName]))
        {
            foreach( ['sync', 'async'] as $type )
            {
                $this->__preSortListeners($eventName, $type);
            }
        }
    }

    private function __preSortListeners($eventName, $type)
    {
        if( $this->listeners[$eventName][$type] )
        {
            krsort($this->listeners[$eventName][$type]);
            $this->sorted[$eventName][$type] = call_user_func_array(
                'array_merge', $this->listeners[$eventName][$type]
            );
        }
    }

    /**
     *  返回listener的处理类和处理方法
     *
     * @param  string  $listener
     * @return array
     */
    protected function parseClassCallable($listener)
    {
        $segments = explode('@', $listener);

        return [$segments[0], count($segments) == 2 ? $segments[1] : 'handle'];
    }

    /**
     * Remove a set of listeners from the dispatcher.
     *
     * @param  string  $event
     * @return void
     */
    public function forget($event)
    {
        unset($this->listeners[$event], $this->sorted[$event]);
    }
}

